/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TemplateModelDecorator.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.models;

import org.barracudamvc.core.comp.IterativeModel;
import org.barracudamvc.core.comp.TemplateDirective;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.model.ModelListener;

//import org.barracudamvc.core.event.*;

/** Abstract Decorator class for using cascading models.
 * It implements the well known decorator pattern.
 * This class is subclassed by concrete Decorators, see {@link MapDecorator}.<br>
 * TemplateModelDecorator is capable of handling {@link
 * org.barracudamvc.core.comp.TemplateModel } as well as {@link
 * org.barracudamvc.core.comp.IterativeModel }. <br>
 * <B>Implementation detail:</B>
 * Nearly all calls are passed directly to the decorated model. If the decorated
 * model implements {@link org.barracudamvc.core.comp.IterativeModel }, also
 * all iterative methods are passed through.<br>
 * Subclasses should override getItem, in order to modify the standard behavior.
 * @author Stefan Armbruster
 * @version $Id: TemplateModelDecorator.java 114 2005-12-09 15:51:51Z christianc $
 */
public abstract class TemplateModelDecorator implements TemplateModel, IterativeModel {

    /** reference to the decorated model */
    protected TemplateModel _templateModel;

    public TemplateModelDecorator(TemplateModel tm) {
        _templateModel = tm;
    }

    public String getName() {
        return _templateModel.getName();
    }

    public boolean processDirective(TemplateDirective td) {
        return _templateModel.processDirective(td);
    }

    //csc_030703.1 - added
    public Object getItem(TemplateDirective td) {
        return _templateModel.getItem(td);
    }

//    abstract public Object getItem(String key);



    // Implementation of Model interface
    public void addModelListener(ModelListener ml)  {
        _templateModel.addModelListener(ml);
    }

    public void removeModelListener(ModelListener ml) {
     _templateModel.removeModelListener(ml);
    }

    // implementation of Contextual interface
    public ViewContext getViewContext() {
        return _templateModel.getViewContext();
    }

    public void setViewContext(ViewContext vc) {
        _templateModel.setViewContext(vc);
    }

    // implementation of IterativeModel

    private boolean isIterativeModel() {
        return _templateModel instanceof IterativeModel;
    }

    public boolean hasNext() {
        if (isIterativeModel()) {
            return ((IterativeModel)_templateModel).hasNext();
        } else {
            return false;
        }
    }

    public void loadNext() {
        if (isIterativeModel()) {
            ((IterativeModel)_templateModel).loadNext();
        }
    }

    public void postIterate() {
        if (isIterativeModel()) {
            ((IterativeModel)_templateModel).postIterate();
        }
    }

    public void preIterate() {
        if (isIterativeModel()) {
            ((IterativeModel)_templateModel).preIterate();
        }
    }

}
