/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AbstractListModel.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.comp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.barracudamvc.core.comp.model.ModelListener;

/**
 * This class provides the abstract implementation
 * for a List Model.
 */
public abstract class AbstractListModel implements ListModel {

    protected ViewContext viewContext = null;
    protected List listeners = new ArrayList();

    //--------------- AbstractTemplateModel ----------------------
    /**
     * Add a listener to the template that's notified each time a change
     * to the data model occurs.
     *
     * @param ml the TemplateModelListener
     */
    public void addModelListener(ModelListener ml) {
        listeners.add(ml);
    }

    /**
     * Remove a listener
     *
     * @param ml the TemplateModelListener
     */
    public void removeModelListener(ModelListener ml) {
        listeners.remove(ml);
    }
    
    /**
     * Forwards the given notification event to all
     * <code>TemplateModelListeners</code> that registered
     * themselves as listeners for this template model.
     */
    public void fireModelChanged() {
        Iterator it = listeners.iterator();
        ModelListener ml = null;
        while (it.hasNext()) {
            ml = (ModelListener) it.next();
            ml.modelChanged(this);
        }
    }
//    protected List listeners = new ArrayList();

    //--------------- AbstractListModel --------------------------
    /**
     * Add a listener to the list that's notified each time a change
     * to the data model occurs.
     *
     * @param l the ListDataListener
     */  
//    public void addListDataListener(ListDataListener l) {
//        listeners.add(l);
//    }

    /**
     * Remove a listener from the list that's notified each time a 
     * change to the data model occurs.
     * @param l the ListDataListener
     */  
//    public void removeListDataListener(ListDataListener l) {
//        listeners.remove(l);
//    }

    /**
     * Reset the model to its initial (unprocessed) state. This 
     * is a convenience method that gets invoked prior to the 
     * entire model being rendered. You only need to override this 
     * method if you want to do something (like reset internal counters)
     * before the model is queried
     */
    public void resetModel() {
        //nop    
    }

    /**
     * AbstractListModel subclasses must call this method <b>after</b>
     * one or more elements of the list change.  The changed elements
     * are specified by a closed interval index0, index1, i.e. the
     * range that includes both index0 and index1.  Note that
     * index0 need not be less than or equal to index1.
     * 
     * @param source The ListModel that changed, typically "this".
     * @param index0 One end of the new interval.
     * @param index1 The other end of the new interval.
     * @see DefaultListModel
     */
/*     
    protected void fireContentsChanged(Object source, int index0, int index1) {
        Iterator it = listeners.iterator();
        ListDataListener ldl = null;
        ListDataEvent e = null;
        while (it.hasNext()) {
            ldl = (ListDataListener) it.next();
            if (e==null) e = new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED, index0, index1);
            ldl.contentsChanged(e);
        }
    }
*/
    /**
     * AbstractListModel subclasses must call this method <b>after</b>
     * one or more elements are added to the model.  The new elements
     * are specified by a closed interval index0, index1, i.e. the
     * range that includes both index0 and index1.  Note that
     * index0 need not be less than or equal to index1.
     * 
     * @param source The ListModel that changed, typically "this".
     * @param index0 One end of the new interval.
     * @param index1 The other end of the new interval.
     * @see DefaultListModel
     */
/*     
    protected void fireIntervalAdded(Object source, int index0, int index1) {
        Iterator it = listeners.iterator();
        ListDataListener ldl = null;
        ListDataEvent e = null;
        while (it.hasNext()) {
            ldl = (ListDataListener) it.next();
            if (e==null) e = new ListDataEvent(source, ListDataEvent.INTERVAL_ADDED, index0, index1);
            ldl.contentsChanged(e);
        }
    }
*/
    /**
     * AbstractListModel subclasses must call this method <b>after</b>
     * one or more elements are removed from the model.  The new elements
     * are specified by a closed interval index0, index1, i.e. the
     * range that includes both index0 and index1.  Note that
     * index0 need not be less than or equal to index1.
     * 
     * @param source The ListModel that changed, typically "this".
     * @param index0 One end of the new interval.
     * @param index1 The other end of the new interval.
     * @see DefaultListModel
     */
/*     
    protected void fireIntervalRemoved(Object source, int index0, int index1) {
        Iterator it = listeners.iterator();
        ListDataListener ldl = null;
        ListDataEvent e = null;
        while (it.hasNext()) {
            ldl = (ListDataListener) it.next();
            if (e==null) e = new ListDataEvent(source, ListDataEvent.INTERVAL_REMOVED, index0, index1);
            ldl.contentsChanged(e);
        }
    }
*/
    /**
     * Return all the listeners for this given list model (returns 
     * a copy of the underlying List)
     *
     * @returns all the listeners for this given list model
     */
/*
    public List getListeners() { 
        return new ArrayList(listeners); 
    }
*/

    //--------------- Contextual ---------------------------------
    /**
     * Specify the ViewContext. This method will generally be called
     * by the class that is using the model to actually render the data
     * in a view. The context will be specified prior to a render pass,
     * and the context will be reset to null after the render pass. 
     *
     * @param ivc the current ViewContext
     */
    public void setViewContext(ViewContext ivc) {
        viewContext = ivc;
    }

    /**
     * Get the current ViewContext
     *
     * @return the current ViewContext
     */
    public ViewContext getViewContext() {
        return viewContext;
    }

}