/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Christian Cryder, Diez B. Roggisch
 *
 * $Id: FormElement.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import java.text.*;

/**
 * <p>This interfaces defines the methods a class needs to implement
 * to act as a FormElement.
 *
 * <p>A FormElement defines how an element in a FormMap should
 * be mapped to a first class java object. There are several
 * key pieces of information required:
 *
 * <ul>
 *         <li>key - used to retrieve the target value from the incoming
 *            data source (either a ServletRequest or a StateMap)</li>
 *         <li>type - the type of data object the incoming value should be
 *            mapped to, as defined by the FormType class (String, Boolean,
 *            Integer, Date, Long, Short, Double, Float)</li>
 *         <li>default val - the default val to be used if the key value is
 *            null in the incoming data source (may be null)</li>
 *         <li>validator - any validators associated with this particular
 *            form elmenent (may be null)</li>
 *         <li>allow multiples - the servlet interface allows multiple params
 *            with the same key. This value indicates whether or not that is
 *            acceptable for a key. If so, then when multiple values are
 *            encountered they element will actually contain a List of values
 *            of the proper type</li>
 * </ul>
 *
 * <p>Once we have this information, we basically have enough data to map
 * a String value in the data source to a first class Java object in the
 * element.
 *
 * <p>Note that you can optionally specify a Format object, which will be used
 * by FormUtil.repopulate() and FormUtil.formatForOutput() to render the form
 * value in a manner that will be fit for human consumption (as well as subsequent
 * form reparsing).
 *
 * <p>Also note that all of the setters have been modified to return a reference
 * to the FormElement. This makes it possible to chain your calls on a single line, 
 * like this:<br><br>
 *
 *           new DefaultFormElement(...).setName("foo").setDefaultVal("blah")
 */
public interface FormElement extends Comparable {

    public FormElement setKey(String key);
    public String getKey();
    public FormElement setName(String name);
    public String getName();
    public FormElement setType(FormType type);
    public FormType getType();
    public FormElement setDefaultVal(Object defaultVal);
    public Object getDefaultVal();
    public FormElement setAllowMultiples(boolean val);
    public boolean allowMultiples();
    public FormElement setValidator(FormValidator validator);
    public FormValidator getValidator();
    public FormElement setOrigVal(Object iorigVal);
    public Object getOrigVal();
    public FormElement setVal(Object ival);
    public Object getVal();
    public Object getVal(Object dflt);
    public Object[] getVals();
    public Object[] getVals(Object[] dflt);
    public FormElement setParseException(ParseException pe);
    public ParseException getParseException();
    public FormElement setFormat(Format iformat);               //csc_122904_1
    public Format getFormat();                                  //csc_122904_1
    public FormElement setParentForm(FormMap iparent);
    public FormMap getParentForm();
//    public String getStringVal();
//    public String getStringVal(String dflt);
//    public Boolean getBooleanVal();
//    public Boolean getBooleanVal(Boolean dflt);
//    public Integer getIntegerVal();
//    public Integer getIntegerVal(Integer dflt);
//    public Date getDateVal();
//    public Date getDateVal(Date dflt);
//    public Long getLongVal();
//    public Long getLongVal(Long dflt);
//    public Short getShortVal();
//    public Short getShortVal(Short dflt);
//    public Double getDoubleVal();
//    public Double getDoubleVal(Double dflt);
//    public Float getFloatVal();
//    public Float getFloatVal(Float dflt);
}
