/*
 * Copyright (C) 2003  Shawn Wilson [shawnw@atmreports.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: CommaSeparatedDOMWriter.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.util.dom;

// java imports:
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;

import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.html.HTMLDocument;

/**
 * @author  shawnw@atmreports.com
 * @since   saw_082603_2
 */
public class CommaSeparatedDOMWriter implements DOMWriter {

    //standard defs
    private static final Class CLASS = CommaSeparatedDOMWriter.class;
    private static final Logger logger = Logger.getLogger(CLASS);

    public static final String eol = System.getProperty("line.separator");
    
    //node definitions
    public static final String DOCUMENT_TYPE = "table";
    public static final String ELEMENT_ROW = "tr";
    public static final String ELEMENT_HEADER = "th";
    public static final String ELEMENT_COLUMN = "td";
    public static final String ELEMENT_SPAN = "span";
    public static final String ELEMENT_DIV = "div";


    //instance objects
    protected String contentType = "text/plain";
    protected String contentDisposition = null;
    protected boolean preventCaching = false;
    protected boolean leaveWriterOpen = false;
    protected int maxAge = 0;

    /**
     * Create a new CommaSeparatedDOMWriter using the default values.
     * The default content type is "text/plain".
     * The default content disposition is to use none;
     */
    public CommaSeparatedDOMWriter() {
        this(null, null);
    }

    /**
     * Create a new CommaSeparatedDOMWriter with the supplied values.
     *
     * @param   icontentType            The content type to use
     * @param   icontentDisposition     The content disposition to use
     */
    public CommaSeparatedDOMWriter(String icontentType, String icontentDisposition) {
        logger.info("Instantiating DOMWriter: "+this);
        setContentType(icontentType);
        setContentDisposition(icontentDisposition);
    }

//csc_012804_1_start
    /**
     * Set the content type (defaults to "text/html" or "text/xml" depending on the
     * document type
     */
    public void setContentType(String icontentType) {
        contentType = icontentType;
    }

    /**
     * Get the content type
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * Set the content disposition (ie. "inline; filename=foo.txt", defaults to null)
     */
    public void setContentDisposition(String icontentDisposition) {
        contentDisposition = icontentDisposition;
    }

    /**
     * Get the content disposition
     */
    public String getContentDisposition() {
        return contentDisposition;
    }

    /**
     * Set whether or not to leave the writer open after writing
     */
    public void setLeaveWriterOpen(boolean val) {
        leaveWriterOpen = val;
    }

    /**
     * Return true if the writer is configured to leave the output stream open
     */
    public boolean getLeaveWriterOpen() {
        return leaveWriterOpen;
    }


    /**
     * Prepare the response object
     *
     * @param node the DOM node to be written out
     * @param resp the HttpServletResponse object
     */
    public void prepareResponse(Node node, HttpServletResponse resp) throws IOException {
        //saw_031604_1 - set a default content type if none is specified
        if (getContentType()==null) setContentType((node instanceof HTMLDocument) ? "text/html" : "text/xml");
        
        //set the content type and disposition
        if (logger.isDebugEnabled()) logger.debug("Setting content type:"+contentType+" content disposition:"+contentDisposition);
        if (contentType!=null) resp.setContentType(contentType);
        if (contentDisposition!=null) resp.setHeader("Content-Disposition", contentDisposition);

        //if we need to prevent caching
        if (preventCaching) {
            //add the appropriate headers to the response
            if (logger.isDebugEnabled()) logger.debug("updating resp hdr to prevent caching");
            resp.setHeader("Pragma","no-cache");
            resp.setHeader("Cache-Control","no-cache");
            resp.setDateHeader("Expires", System.currentTimeMillis());

        //csc_061202.1 - added
        //otherwise explicitly give it a max-age (this will generally allow browsers like
        //IE to page back in history without reloading, but if the user actually revisits the
        //URL, then it will still be reloaded)
        } else {
            resp.setHeader("Cache-Control","max-age="+maxAge);
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());
        }
    }
//csc_012804_1_end

    /**
     * Write a DOM to a ServletResponse object.
     * This method will automatically set the content type for you.
     *
     * @param   node    The DOM node to be written out
     * @param   resp    The HttpServletResponse object to write to
     */
    public void write(Node node, HttpServletResponse resp) throws IOException {
//csc_012804_1_start
/*
        if (contentType!=null) resp.setContentType(contentType);
        if (contentDisposition!=null) resp.setHeader("Content-Disposition", contentDisposition);

        //if we need to prevent caching
        / *
            //add the appropriate headers to the response
            if (logger.isDebugEnabled()) logger.debug("updating resp hdr to prevent caching");
            resp.setHeader("Pragma","no-cache");
            resp.setHeader("Cache-Control","no-cache");
            resp.setDateHeader("Expires", System.currentTimeMillis());
         * /
        resp.setHeader("Cache-Control", "max-age=0");
        resp.setDateHeader("Last-Modified", System.currentTimeMillis());
*/
//saw_031604_1        if (getContentType()==null) setContentType((node instanceof HTMLDocument) ? "text/html" : "text/xml");
        prepareResponse(node, resp);
//csc_012804_1_end

        //write the dom
        
      //saw_031604_1 begin
        // We want to use resp.getOutputStream() instead of resp.getWriter() because the PrintWriter
        // returned by getWriter() apparently has no buffer whereas the ServletOutputStream does.
        // This means that using the PrintWriter would cause the message chunks in the response to
        // be extremely small, with each call to print() causing a new chunk to be sent.
        
//        write(node, resp.getWriter());
        write(node, new OutputStreamWriter(resp.getOutputStream()));
      //saw_031604_1 end
    }

    /**
     * Write a DOM to an OutputStream.
     *
     * @param   node    The DOM node to be written out
     * @param   out     The OutputStream to be written to
     */
    public void write(Node node, OutputStream out) throws IOException {
        write(node, new OutputStreamWriter(out));
    }

    /**
     * Write a DOM to a Writer.
     *
     * @param   node    The DOM node to be written out
     * @param   writer  The writer to be written to
     */
    public void write(Node node, Writer writer) throws IOException {
        try { writeNode(node, writer); }
        catch (DOMException e) {
            logger.error("Error while writing node", e);
        }
        if (!leaveWriterOpen) writer.close();
    }

    protected void writeNode(Node node, Writer writer) throws IOException, DOMException {
        if (logger.isDebugEnabled()) logger.debug("writing node..."+node);
        if (node==null) {
            return;

        //...Element
        } else if (node instanceof Element) {
            String name = node.getNodeName();
            if (logger.isDebugEnabled()) logger.debug("el: "+name);
            if (name.equals(ELEMENT_ROW)) {
                writeRow(node, node.getChildNodes(), writer);
            } else if (name.equals(ELEMENT_HEADER)) {
                writeHeader(node, node.getChildNodes(), writer);
            } else if (name.equals(ELEMENT_COLUMN)) {
                writeColumn(node, node.getChildNodes(), writer);
            } else {
                writeAll(node.getChildNodes(), writer);
            }

        //...CharacterData
        } else if (node instanceof CharacterData) {
            if (!(node instanceof Comment)) {
                if (logger.isDebugEnabled()) logger.debug("char data: "+node.getNodeValue());
                writer.write(node.getNodeValue());
            } else {
                if (logger.isDebugEnabled()) logger.debug("comments (skipping)");
            }

        //...Document
        } else if (node instanceof Document) {
            if (logger.isInfoEnabled()) logger.info("writing document: "+node.getClass());
            writeAll(node.getChildNodes(), writer);

        //...DocumentType
        } else if (node instanceof DocumentType) {
            if (logger.isInfoEnabled()) logger.info("document type: "+node.getNodeName());
            String name = node.getNodeName();
            if (!name.equals(DOCUMENT_TYPE)) {
                throw new DOMException(DOMException.NOT_SUPPORTED_ERR, "Document type not supported; required \""+DOCUMENT_TYPE+"\", found \""+name+"\"");
            }

        //...child nodes
        } else {
            logger.warn("Unrecognized node type: "+node.getClass());
            writeAll(node.getChildNodes(), writer);
        }
    }

    protected void writeAll(NodeList nodes, Writer writer) throws IOException, DOMException {
        if (logger.isDebugEnabled()) logger.debug("writing all..."+nodes);
        int len = nodes.getLength();
        for (int i=0; i<len; i++) {
            if (Thread.interrupted()) throw new IOException("Thread interrupted while generating output");
            writeNode(nodes.item(i), writer);
        }
    }

    protected void writeRow(Node parent, NodeList nodes, Writer writer) throws IOException, DOMException {
        if (logger.isDebugEnabled()) logger.debug("writing row..."+nodes);
        StringWriter sw = new StringWriter(1000);
        writeAll(nodes, sw);
        sw.close();

//        StringBuffer sb = sw.getBuffer();
////        sb.setCharAt(sb.length()-1, '\n');
//        sb.append(eol); //this may be a problem, because its the server's eol marker - technically we should probably be using the client's eol marker
//        writer.write(sb.toString());
        writer.write(sw.getBuffer().toString());
    }

    protected void writeHeader(Node parent, NodeList nodes, Writer writer) throws IOException, DOMException {
//        if (logger.isDebugEnabled()) logger.debug("writing hdr..."+nodes);
        //for now we just delegate to writeColumn()
        writeColumn(parent, nodes, writer);
    }

    protected void writeColumn(Node parent, NodeList nodes, Writer writer) throws IOException, DOMException {
        if (logger.isDebugEnabled()) logger.debug("writing col..."+nodes);
        StringWriter sw = new StringWriter(100);
        writeAll(nodes, sw);
        sw.close();

        String data = sw.toString();
        if (parent.getPreviousSibling()!=null) writer.write(",");   //if our parent (the row) is NOT the first sibling, prepend on a comma
        writer.write('"');                                          //leading quote
        writer.write(data.replaceAll("\"", "\"\""));                //change any embedded quotes to double quotes
        writer.write('"');                                          //trailing quote
        if (parent.getNextSibling()==null) writer.write(eol);       //if our parent (the row) is the LAST row, write an eol - NOTE: this may be a problem, because its the server's eol marker - technically we should probably be using the client's eol marker
    }

}
