/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MainScreen.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.examples.ex1;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BAction;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.DispatchQueue;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.ListenerFactory;
import org.barracudamvc.core.event.helper.BTemplateViewHandler;
import org.barracudamvc.core.event.helper.EventForwardingFactory;
import org.barracudamvc.core.util.http.SessionServices;
import org.barracudamvc.examples.ex1.events.ExpireSession;
import org.barracudamvc.examples.ex1.events.GetLoginScreen;
import org.barracudamvc.examples.ex1.events.GetMainScreen;
import org.barracudamvc.examples.ex1.events.GetMainScreenSlowly;
import org.barracudamvc.examples.ex1.events.RenderMainScreen;
import org.barracudamvc.examples.ex1.xmlc.MainHTML;
import org.barracudamvc.plankton.data.ObjectRepository;


/**
 * Event handlers (both Controller and View) for the 
 * Main screen.
 */
public class MainScreen extends DefaultEventGateway {

    //public constants
    protected static final Logger logger = Logger.getLogger(MainScreen.class.getName());

    //this defines the various event handlers
    private ListenerFactory getMainFactory =               new EventForwardingFactory(new RenderMainScreen());
    private ListenerFactory getMainSlowlyFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new GetMainScreenSlowlyHandler();}        public String getListenerID() {return getID(GetMainScreenSlowlyHandler.class);}};                
    private ListenerFactory expireSessionFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new ExpireSessionHandler();}        public String getListenerID() {return getID(ExpireSessionHandler.class);}};                
    private ListenerFactory renderMainFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderMainScreenHandler();}    public String getListenerID() {return getID(RenderMainScreenHandler.class);}};

    /**
     * Public constructor
     */
    public MainScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(getMainFactory, GetMainScreen.class);
        specifyLocalEventInterests(getMainSlowlyFactory, GetMainScreenSlowly.class);
        specifyLocalEventInterests(expireSessionFactory, ExpireSession.class);
        specifyLocalEventInterests(renderMainFactory, RenderMainScreen.class);
    }


    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * GetMainScreenSlowlyHandler - get the main screen slowly
     */
    class GetMainScreenSlowlyHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
        
            //get the LongRunning object so we can update it
//            LongRunning lr = (LongRunning) context.getEvent();
//            lr.setRedirectEvent(new GetMainScreen());
//            lr.setETA(120);
            
            //do something that takes a long time (2 minutes)
            logger.info("GetMainScreenSlowly...");
            for (int i=0; i<15; i++) {
                try {
                    logger.debug("...iteration: "+i);
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    throw new IOException("Running thread was interrupted! Long running process will be terminated...");
                }
            }
            logger.info("done!");
        
            //forward control on to rendering even
            context.getQueue().addEvent(new RenderMainScreen());
        }
    }
    
    /**
     * ExpireSessionHandler - this is where we handle any ExpireSession
     * events. Afterwards, redirect flow to the Main screen (which won't
     * work because the session information is gone now, so instead you'll
     * end up at the Login screen as you really should.)
     */
    class ExpireSessionHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletRequest req = context.getRequest();
            DispatchQueue queue = context.getQueue();
            logger.debug("Expire session");

            //expire the session info
            SessionServices.getSession(req).invalidate();

            //in this case, we're not really doing anything special, so just
            //redirect to the RenderMain view (note, this won't actually happen,
            //because the Utilities class is ensuring that without session info 
            //we always get redirected to the Login screen...but we shouldn't have 
            //to know about that detail here)
            BaseEvent newEvent = new GetMainScreen();
            newEvent.setSource(event);
            queue.addEvent(newEvent);
        }
    }
    
    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderMainScreenHandler - this is where we handle any RenderMainScreen 
     * event and actually generate the view. We check the session info 
     * to determine how many times they've been here, as well as to see
     * if they were automatically logged in.
     */
    class RenderMainScreenHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return MainHTML.class;}
    }
    
    
    //------------------------------------------------------------
    //                Components - TemplateModel
    //------------------------------------------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
        boolean gotState = false;
        int numberOfVisits = -1;
        boolean autoLogin = false;
    
        //register the model by name
        public String getName() {return "Ex1_Main";}

        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Visits")) {
                checkState();
                return numberOfVisits+" time"+(numberOfVisits>1 ? "s" : "");
            } else if (key.equals("AutoLogin")) {
                checkState();
                if (autoLogin) return new BComponent(); //visible (go ahead and show the node)
                else return null;                       //null (skip it)
            } else if (key.equals("SecretTip")) {
                checkState();
                if (numberOfVisits>3) return new BComponent(); //visible (go ahead and show the node)
                else return null;                       //null (skip it)
            } else if (key.equals("GetLoginLink")) {
                return new BAction(new GetLoginScreen());
            } else if (key.equals("GetMainScreenSlowly")) {
                BAction baction = new BAction(new GetMainScreenSlowly());
                baction.setParam("key1","foo");
                baction.setParam("key2","bar");
                baction.setParam("key3","rama");
                return baction;
            } else if (key.equals("ExpireSessionLink")) {
                return new BAction(new ExpireSession());
            } else {
                return super.getItem(key);
            }
        }
        
        private void checkState() {
            if (gotState) return;
            ViewContext vc = getViewContext();
            //...see how many times we've been here...
            HttpSession session = SessionServices.getSession(vc);
            numberOfVisits = 0;
            try {numberOfVisits = Integer.parseInt(""+session.getAttribute(LoginServices.NUMBER_OF_VISITS).toString());}
            catch (Exception e) {}
            session.setAttribute(LoginServices.NUMBER_OF_VISITS, ""+(++numberOfVisits));
            //..see if we auto-logged-in
//csc_082302.2_start
//            autoLogin = new Boolean(""+session.getAttribute(LoginServices.AUTO_LOGIN)).booleanValue();
            ObjectRepository lr = ObjectRepository.getLocalRepository();
            Boolean b = (Boolean) (lr.getState(LoginServices.AUTO_LOGIN));
            autoLogin = (b!=null && b.booleanValue());
//csc_082302.2_end
            gotState = true;
        }
    }
}

