/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DefaultStateMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.plankton.data;

import java.util.*;

import org.apache.log4j.Logger;

/**
 * <p>A StateMap is an object that is capable of carrying state 
 * information along with it--you can put properties into the 
 * state and then get them back out.
 *
 * <p>This class provides the default implementation of a StateMap. 
 * The underlying storage structure is a HashMap, so it a) isn't 
 * threadsafe and b) accepts nulls. If you need more than the minimal 
 * functionality exposed in the StateMap interface, you should work 
 * with the underlying Map structure.
 *
 * <p>Key entities that implement StateMap:
 * <ul>
 *        <li>BaseEvent</li>
 *        <li>EventContext</li>
 *        <li>FormMap</li>
 * </ul>
 */
public class DefaultStateMap implements StateMap {

    //public vars
    protected static final Logger logger = Logger.getLogger(DefaultStateMap.class.getName());

    //private vars
    protected Map props = null;                    //private property map

    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        if (props==null) props = new HashMap();
        if (logger.isDebugEnabled()) logger.debug("Setting queue state: [key]:"+key+" [val]:"+val);
        props.put(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (props==null) return null;
        else return props.get(key);
    }
    
    /**
     * remove a property in this StateMap. This function was expanded in csc_101803_2 to
     * support the notion of wildcarding, allowing you to remove multiple keys in one fell
     * swoop. Basically, if the key is a String, which ends with an '*', then any keys that
     * start with that string will be removed (and in this case, the method returns a Map of
     * key/val pairs that got removed, rather than a single object that got removed). This 
     * approach is not quite as flexible as using regular expressions, but that would make us 
     * dependent on jdk1.4 (so we won't go there for now). Note that this class backs the 
     * ObjectRepository data structures, so this functionality applies there as well.
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        //eliminate the obvious
        if (props==null) return null;
        
        //remove all keys that match
        if (key!=null && (key instanceof String) && ((String) key).endsWith("*")) {
            Map removed = new HashMap();
            String keystr = (String) key;
            String targetstr = keystr.substring(0,keystr.length()-1);
//csc_101304_1_start
/*
            List keys = getStateKeys();
            Iterator it = keys.iterator();
            while (it.hasNext()) {
                Object okey = it.next();
                if ((okey instanceof String) && ((String) okey).startsWith(targetstr)) {
                    Object oval = props.get(okey);
                    removed.put(okey, oval);
                    props.remove(okey);
                }
            }
*/
/*
throws a ConcurrentModificationException
            Set entries = props.entrySet();
            Iterator it = entries.iterator();
            while (it.hasNext()) {
                Map.Entry me = (Map.Entry) it.next();
                if ((""+me.getKey()).startsWith(targetstr)) {
                    removed.put(me.getKey(), me.getValue());
                    entries.remove(me);
                }
            }
*/            
            List keys = new ArrayList(getStateKeys());
            Iterator it = keys.iterator();
            while (it.hasNext()) {
                Object okey = it.next();
                if ((okey instanceof String) && ((String) okey).startsWith(targetstr)) {
                    Object oval = props.remove(okey);
                    removed.put(okey, oval);
                }
            }
//csc_101304_1_end            
            return (removed.size()>0 ? removed : null);        
        } else {
            return props.remove(key);
        }
    }
    
    /**
     * get a keyset for this StateMap (whether or 
     * not the set is backed by the data store depends on 
     * the implementation). In this particular case, the set
     * is backed by a copy of the underlying data structure
     *
     * @return a Set of keys for this StateMap
     */
    public Set getStateKeys() {
//csc_101304_1        if (props==null) return null;
//csc_101304_1        else return new ArrayList(props.keySet());
        if (props==null) return new HashSet();      //csc_101304_1
//csc_110504_1        else return props.keySet();                 //csc_101304_1
        else return new HashSet(props.keySet());    //csc_110504_1
    }
    
    /**
     * get a Map that holds the state values (whether or 
     * not the Map is backed by the data store depends on 
     * the implementation). In this particular case, the map
     * IS the underlying data store itself
     *
     * @return a Map of key/val pairs for this StateMap
     */
    public Map getStateStore() {
//csc_101304_1        if (props==null) props = new HashMap();
//csc_101304_1        return new HashMap(props);
        if (props==null) props = new HashMap();     //csc_101304_1
        return props;                               //csc_101304_1
    }

    /**
     * clear all state information
     */
    public void clearState() {
        if (props!=null) props.clear();
    }

}
